package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.GetQuestionInfoDTO;
import com.diagbot.dto.QuestionIndexDTO;
import com.diagbot.dto.QuestionPageDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.QuestionDetail;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.entity.QuestionMapping;
import com.diagbot.entity.wrapper.QuestionWrapper;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.QuestionDetailService;
import com.diagbot.service.QuestionMappingService;
import com.diagbot.service.impl.QuestionInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.GetQuestionInfoVO;
import com.diagbot.vo.GetQuestionUsualByDeptVO;
import com.diagbot.vo.QuestionPageVO;
import com.diagbot.vo.QuestionSaveVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/23 11:37
 */
@Component
public class QuestionInfoFacade extends QuestionInfoServiceImpl {

    @Autowired
    QuestionDetailFacade questionDetailFacade;
    @Autowired
    @Qualifier("questionDetailServiceImpl")
    QuestionDetailService questionDetailService;
    @Autowired
    @Qualifier("questionMappingServiceImpl")
    QuestionMappingService questionMappingService;
    @Autowired
    QuestionMappingFacade questionMappingFacade;
    @Autowired
    ModuleDetailFacade moduleDetailFacade;


    /**
     * 标签保存
     *
     * @param questionSaveVO
     */
    public void saveOrUpdate(QuestionSaveVO questionSaveVO) {
        QuestionWrapper questionWrapper = questionSaveVO.getQuestionWrapper();
        CommonParam param = initCommonParam();
        saveQuestionInfo(questionWrapper, param);
        switch (questionWrapper.getTagType()) {
            case "1":   //单项，例如：程度、体温
                saveQuestionDetail(questionWrapper, param);
                break;
            case "2":   //组合项，例如：杂音，修改主表和mapping
                saveQuestionMapping(questionWrapper, param);
                break;
            case "3":
                break;
            case "4":
                break;
            case "5":
                break;
            case "6":
                break;
            case "7":
                break;
            case "8":
                break;
            default:
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "请选择标签显示类型");
        }
    }


    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        //String person = UserUtils.getCurrentPrincipleID(); //TODO person
        String person = "1";
        param.setNow(DateUtil.now());
        param.setPerson(person);
        param.setSaveOrUpdate("save");
        QuestionInfo questionInfo = new QuestionInfo();
        param.setQuestionInfo(questionInfo);
        return param;
    }


    /**
     * 保存映射关系
     *
     * @param questionWrapper
     * @param param
     */
    public void saveQuestionMapping(QuestionWrapper questionWrapper, CommonParam param) {
        QuestionInfo questionInfo = param.getQuestionInfo();
        Date now = param.getNow();
        String person = param.getPerson();
        //先删除原明细，再插入新明细
        questionMappingFacade.update(new QuestionMapping(), //删除原映射关系
                new UpdateWrapper<QuestionMapping>()
                        .eq("parent_question", questionInfo.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("gmt_modified", now)
                        .set("modifier", person)
                        .set("is_deleted", IsDeleteEnum.Y.getKey()));
        List<QuestionMapping> questionMappings = questionWrapper.getQuestionMappings();
        if (ListUtil.isNotEmpty(questionMappings)) {
            List<QuestionMapping> saveMapping = new ArrayList<>();
            int i = 1;
            for (QuestionMapping mapping : questionMappings) {
                QuestionMapping bean = new QuestionMapping();
                BeanUtil.copyProperties(mapping, bean);
                bean.setId(null); //防止前端传参，将前端的id置空自动插入
                bean.setCreator(person);
                bean.setGmtCreate(now);
                bean.setModifier(person);
                bean.setGmtModified(now);
                bean.setOrderNo(i++);
                saveMapping.add(bean);
            }
            questionMappingService.saveBatch(saveMapping);
        }
    }


    /**
     * 保存明细
     *
     * @param questionWrapper
     * @param param
     */
    public void saveQuestionDetail(QuestionWrapper questionWrapper, CommonParam param) {
        QuestionInfo questionInfo = param.getQuestionInfo();
        Date now = param.getNow();
        String person = param.getPerson();
        //先删除原明细，再插入新明细
        questionDetailFacade.update(new QuestionDetail(), //删除原明细
                new UpdateWrapper<QuestionDetail>()
                        .eq("question_id", questionInfo.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .set("gmt_modified", now)
                        .set("modifier", person)
                        .set("is_deleted", IsDeleteEnum.Y.getKey()));
        List<QuestionDetail> questionDetails = questionWrapper.getQuestionDetails();
        if (ListUtil.isNotEmpty(questionDetails)) {
            List<QuestionDetail> saveDetail = new ArrayList<>();
            int i = 1;
            for (QuestionDetail detail : questionDetails) {
                QuestionDetail bean = new QuestionDetail();
                BeanUtil.copyProperties(detail, bean);
                bean.setId(null); //防止前端传参，将前端的id置空自动插入
                bean.setCreator(person);
                bean.setGmtCreate(now);
                bean.setModifier(person);
                bean.setGmtModified(now);
                bean.setQuestionId(questionInfo.getId());
                bean.setOrderNo(i++);
                saveDetail.add(bean);
            }
            questionDetailService.saveBatch(saveDetail);
        }
    }


    /**
     * 保存主表
     *
     * @param questionWrapper
     * @param param
     */
    public void saveQuestionInfo(QuestionWrapper questionWrapper, CommonParam param) {
        QuestionInfo questionInfo = param.getQuestionInfo();
        if (questionWrapper.getId() != null) {
            questionInfo = this.getOne(new QueryWrapper<QuestionInfo>()
                    .eq("id", questionWrapper.getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey()));
            if (questionInfo == null) { //校验
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标签不存在");
            }
            param.setSaveOrUpdate("update");
        }
        BeanUtil.copyProperties(questionWrapper, questionInfo);
        List<QuestionInfo> questionInfoList = this.list(new QueryWrapper<QuestionInfo>()
                .eq("tag_name", questionInfo.getTagName())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("type", questionInfo.getType())
                .ne("id", questionInfo.getId() == null ? -1 : questionInfo.getId()));
        if (questionInfoList.size() > 0) { //标签type、tagName唯一
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标签名已存在");
        }
        if ("save".equals(param.getSaveOrUpdate())) {
            questionInfo.setCreator(param.getPerson()); //创建人
            questionInfo.setGmtCreate(param.getNow());//创建时间
        }
        questionInfo.setGmtModified(param.getNow());//修改时间
        questionInfo.setModifier(param.getPerson());//修改人
        this.saveOrUpdate(questionInfo);
    }

    /**
     * 标签列表
     *
     * @param questionPageVO
     * @return
     */
    public IPage<QuestionPageDTO> getListFac(QuestionPageVO questionPageVO) {

        IPage<QuestionPageDTO> res = this.getList(questionPageVO);
        //        List<QuestionPageDTO> data = res.getRecords();
        return res;
    }


    /**
     * 根据id删除标签
     *
     * @param ids
     * @return
     */
    public Boolean deleteByIdsFac(String ids) {

        if (StringUtil.isEmpty(ids)) {
            return true;
        }

        Map paramMap = new HashMap<>();
        paramMap.put("delete", IsDeleteEnum.Y.getKey());
        paramMap.put("ids", Arrays.asList(ids.split(",")));
        paramMap.put("gmtModified", DateUtil.now());
        paramMap.put("modifier", UserUtils.getCurrentPrincipleID());

        //删除自身
        this.deleteByIds(paramMap);

        //删除明细
        questionDetailFacade.deleteByQuestionIdFac(paramMap);

        //删除映射关系
        questionMappingFacade.deleteByIdsFac(paramMap);

        //删除模板中的标签
        moduleDetailFacade.deleteByQuestionIdFac(paramMap);
        return true;
    }


    /**
     * 检索
     *
     * @param questionIndexDTO
     * @return
     */
    public List<QuestionInfo> indexFac(QuestionIndexDTO questionIndexDTO) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("isDeleted", IsDeleteEnum.N.getKey());
        paramMap.put("age", questionIndexDTO.getAge());
        paramMap.put("sexType", questionIndexDTO.getSexType());
        paramMap.put("tagName", questionIndexDTO.getTagName());
        paramMap.put("type", questionIndexDTO.getType());
        return this.index(paramMap);
    }

    /**
     * 根据标签名获取标签信息
     *
     * @param questionInfo
     * @return
     */
    public QuestionInfo getQuestionInfos(QuestionInfo questionInfo) {
        QueryWrapper<QuestionInfo> queryWrapper = new QueryWrapper<QuestionInfo>();
        queryWrapper.eq("is_deleted", "N");
        queryWrapper.eq("type", 1);
        queryWrapper.eq("name", questionInfo.getName());
        return this.getOne(queryWrapper);
    }

    /**
     * 根据标签名获取标签信息
     *
     * @param getQuestionInfoVO
     * @return
     */
    public List<GetQuestionInfoDTO> getQuestiongInfosByName(GetQuestionInfoVO getQuestionInfoVO) {
        List<GetQuestionInfoDTO> getQuestionInfoDTOS = this.getQuestiongInfoByName(getQuestionInfoVO);
        return getQuestionInfoDTOS;
    }

    /**
     * 根据科室id获取常用标签信息
     *
     * @param getQuestionUsualByDeptVO
     * @return
     */
    public List<GetQuestionInfoDTO> getQuestionUsualByDept(GetQuestionUsualByDeptVO getQuestionUsualByDeptVO) {
        List<GetQuestionInfoDTO> getQuestionInfoDTOList = this.getQuestionUsualsByDept(getQuestionUsualByDeptVO);
        return getQuestionInfoDTOList;
    }
}
