package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.AddTagRetrievalDTO;
import com.diagbot.dto.GetRetrievalsByTagDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.RetrievalFacade;
import com.diagbot.vo.AddTagRetrievalVO;
import com.diagbot.vo.DelRetrievalsByMapsVO;
import com.diagbot.vo.GetRetrievalsByTagVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 同义词检索表 前端控制器
 * </p>
 *
 * @author rgb
 * @since 2018--12
 */
@RestController
@RequestMapping("/retrieval")
@SuppressWarnings("unchecked")
@Api(value = "同义词维护相关API", tags = { "同义词维护相关API" })
public class RetrievalController {

    @Autowired
    private RetrievalFacade retrievalFacade;

    @ApiOperation(value = "根据标签获取同义词[by:rengb]")
    @PostMapping("/getRetrievalsByTag")
    @SysLogger("getRetrievalsByTag")
    public RespDTO<List<GetRetrievalsByTagDTO>> getRetrievalsByTag(@Valid @RequestBody GetRetrievalsByTagVO getRetrievalsByTagVO) {
        return RespDTO.onSuc(retrievalFacade.getRetrievalsByTag(getRetrievalsByTagVO));
    }


    @ApiOperation(value = "根据映射关系删除标签的同义词[by:rengb]")
    @PostMapping("/delRetrievalsByMaps")
    @SysLogger("delRetrievalsByMaps")
    @Transactional
    public RespDTO<List<Boolean>> delRetrievalsByMaps(@Valid @RequestBody DelRetrievalsByMapsVO delRetrievalsByMapsVO) {
        return RespDTO.onSuc(retrievalFacade.delRetrievalsByMaps(delRetrievalsByMapsVO));
    }

    @ApiOperation(value = "标签添加同义词[by:rengb]")
    @PostMapping("/addTagRetrieval")
    @SysLogger("addTagRetrieval")
    @Transactional
    public RespDTO<AddTagRetrievalDTO> addTagRetrieval(@Valid @RequestBody AddTagRetrievalVO addTagRetrievalVO) {
        return RespDTO.onSuc(retrievalFacade.addTagRetrieval(addTagRetrievalVO));
    }


}
